from django.db.models import Q
import re
from tastypie.resources import ModelResource , ALL , ALL_WITH_RELATIONS
from tastypie.authorization import Authorization
from tastypie import fields
from apps.pregunta.models import * 
from apps.organizacion.models import * 
from apps.usuario.models import  Settings , Rol , Suscripcion
from django.core.urlresolvers import reverse
from tastypie.paginator import Paginator
from django.template.defaultfilters import date
from tastypie.paginator import Paginator
from django.contrib.auth.models import User
from tastypie.exceptions import BadRequest
#from apps.api.helpers import helper
# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ******************************************* SEARCH QUESTIONS API ********************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************
PATH_URL_IMG = "/media/img/unknown.png"
 
class Handlers:

	""" regresa los contactos que tienes permitido ver , los que estan dentro de x organizacion o 
	contienen a alguien en su interaccion"""
	def get_allow_contacts(self , id):
		
		user_id =  int(id)
		#obtener a que organizacion pertenece el usuario
		list_own_organizations = organizacion_usuario.objects.filter(miembro = user_id).values("organizacion")
		#obtener todos los miembros de la organizaicon a traves de la lista de ids_organizacion menos el mismo usuario , porque regresa todos los miembros de la organizacion inclruyendote
		list_own_contacts = organizacion_usuario.objects.all().filter(organizacion__in = list_own_organizations ).exclude(miembro=user_id).values("miembro")
		#obtenemos todas las instancias de los usuarios
		all_users_contact = User.objects.filter( id__in = list_own_contacts).exclude(id=user_id)

		#obtenemos todos los usuarios a los que esta suscrito este usuario
		list_user_suscribe = Suscripcion.objects.filter(suscriptor = user_id).values("suscrito_to")
		#obtenemos todas las instancias de los usuarios suscritos
		all_users_suscribe  = User.objects.filter(id__in = list_user_suscribe ).exclude(id=user_id)

		return all_users_contact | all_users_suscribe 


class SearchQuestionResource(ModelResource):
	"""
	Return - Todas las busquedas que coincidan con los parametros para buscar preguntas por informacion , por pregunta titulo y folio  
		 y tengas permitido ver , si eres admim puedes ver todo , si eres otro no.
	"""
	de = fields.ForeignKey("apps.api.resource.UsuarioResource",  full = True , attribute = 'de')
	para = fields.ForeignKey("apps.api.resource.UsuarioResource", 'para'    , full = True )
	status_id = fields.ForeignKey("apps.api.resource.StatusResource", 'status_id' ,  full =True )
	informacion = fields.CharField(attribute='informacion')


	class Meta:

		queryset = Pregunta.objects.all().order_by('-creacion')
		resource_name = 'buscarpregunta'
		excludes = ['resource_uri']	

	def dehydrate(self , bundle):
		return bundle

	def get_object_list(self , request):

		try:
			user_keys =  request.GET.get('query_string')
			
			#obtenemos todos los usuarios que se tiene permitido ver
			handler = Handlers()

			user_id =  int(request.GET.get('user_suscriptor'))

			all_users_allowd = handler.get_allow_contacts(user_id)
			all_users_allowed = all_users_allowd.values("id")
			print all_users_allowed 
			
			user_result  =   super(SearchQuestionResource, self).get_object_list(request).filter( 
				 	    
					    Q(informacion__icontains = user_keys) | Q(pregunta__icontains = user_keys) | Q(folio__iexact= user_keys) , Q(de__in = all_users_allowed ) 
					    )[:5]
			
		
		 	return user_result 
		except:
			return	Suscripcion.objects.all().distinct() 


# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ******************************************* SEARCH USERS FEATURE API **************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************

class SuscripcionResource(ModelResource):
	"""
	Return - Todos los usuarios a los que esta suscrito el suscriptor  ( argument )

	@params - suscriptor -- ID usuario
	"""
#	suscriptor = fields.ForeignKey('apps.api.resource.UsuarioResource' , 'suscriptor')
	suscrito_to = fields.ForeignKey('apps.api.resource.UsuarioResource' , 'suscrito_to' , full = True)

	class Meta:
		queryset = Suscripcion.objects.all().distinct()
		resource_name = 'suscripcion'
		excludes = ['user_suscriptor', 'resource_uri']	

	def dehydrate(self , bundle):
		print "qui todo el bundleee"
		print bundle

		del bundle.data["resource_uri"]
		del bundle.data["id"]
		bundle.data["first_name"] =  bundle.data["suscrito_to"].data["first_name"]
		bundle.data["id"] =  bundle.data["suscrito_to"].data["id"]
		bundle.data["img"] =  bundle.data["suscrito_to"].data["img"]
		bundle.data["last_name"] =  bundle.data["suscrito_to"].data["last_name"]
		bundle.data["resource_uri"] =  bundle.data["suscrito_to"].data["resource_uri"]
		bundle.data["username"] =  bundle.data["suscrito_to"].data["username"]
		del bundle.data["suscrito_to"]

		return bundle

	def get_object_list(self , request):
		
		try:
			user_id =  int(request.GET.get('user_suscriptor'))

			user_suscription =   super(SuscripcionResource, self).get_object_list(request).filter(suscriptor = user_id )
			print user_suscription,"suscripcion"
		
		 	return user_suscription  
		except:
			return	Suscripcion.objects.all().distinct() 
# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ************************************* GET CONTACTS ALLOWED TO SEE API *************************************************
# ***********************************************************************************************************************
#esta clase obtiene todos los usuarios que estan dentro de las organizaciones que stoy incluido y los usurios que estoy suscrito ( interacturando )
#este setting se usa para obtener datos sensibles del usuario , password, email, 
class AllowedContactsResource(ModelResource):

	class Meta:
		queryset = User.objects.all()
		resource_name = 'allow_contacts'
		filtering = {
		}
	def dehydrate(self , bundle ):
		
		setting = Settings.objects.filter(user_id = bundle.obj )
		bundle.data['img'] =	setting[0].img
		bundle.data['rol'] = setting[0].rol.nombre
		return bundle
	
	def get_object_list(self , request):
		
		try:
			user_id =  int(request.GET.get('user_suscriptor'))
			handler = Handlers()

			user_keys =  request.GET.get('user_contact')
			#obtenemos los contactos a los que esta suscrito user_contact 
			all_users_allowed = handler.get_allow_contacts(user_id)

			if user_keys:

				split_data = user_keys.split(" ")
				last_name = split_data[1] if len(split_data ) > 1  else  " "
				first_name = split_data[0]
				print user_keys
				#si no existe un apellido en el sig. formato rocio galaviz entoncs intentamos buscar el apellido con la primera posicion en parametro GET
				#si solo un parametro ha sido ingresado ( solo una palabra )
				if last_name == " ":
					#aqui solo se ingreso una palabra , es posible que quiera buscar por nombre o por apellido
					user_result  =   all_users_allowed.filter(Q(last_name__icontains = first_name ) | Q(first_name__icontains = first_name ) )
				else:
					#aqui se ingreso nombre y apellido o sea dos palabras
					user_result  =   all_users_allowed.filter(Q(last_name__icontains = last_name ) | Q(first_name__icontains = first_name ) )
				#si se filtro el usuario por letra nombre o apellido
				return user_result
		
			return all_users_allowed 
		except:
			return	 User.objects.all()[0]



# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ************************************* SETTTING API ********************************************************************
# ***********************************************************************************************************************
	
#este setting se usa para obtener datos sensibles del usuario , password, email, 
class SettingResource(ModelResource):

	class Meta:
		queryset = User.objects.all()
		resource_name = 'setting'
		#excludes = ['creacion']
		filtering = {
		}
	def dehydrate(self , bundle ):
		
		setting = Settings.objects.filter(user_id = bundle.obj )
		bundle.data['img'] =	setting[0].img
		bundle.data['rol'] = setting[0].rol.nombre
		return bundle


# ******************************************* USUARIO API **************************************************************
# ******************************************* USUARIO API **************************************************************
# ******************************************* USUARIO API **************************************************************
# ******************************************* USUARIO API **************************************************************
# ******************************************* USUARIO API **************************************************************
class UsuarioResource(ModelResource):


	class Meta:
		
		queryset = User.objects.all().order_by("-date_joined")
		resource_name = 'usuario'
		excludes = ['password' , 'last_login','is_superuser', 'email', 'date_joined'  ,'is_active', 'is_staff',]
		filtering = {
			
			"first_name" : ["icontains"],
			"last_name" : ["icontains"],
		}

		#serializer = Serializer()
		authorization= Authorization()
	def dehydrate(self , bundle):
		setting = Settings.objects.filter(user_id = bundle.obj )
		print setting
		bundle.data['img'] =	setting[0].img
#		bundle.data['rol'] = setting[0].rol
		return bundle
# ******************************************* PREGUNTA API **************************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************


#status de la pregunta
class StatusResource(ModelResource):
 
	class Meta:
		#allowed_methods = ['get', 'delete', 'post', 'put']		
		queryset = Status.objects.all()
		resource_name = 'status'
		authorization= Authorization()
 

#contenido general de la pregunta
class PreguntaResource(ModelResource):

	#both usuario_creador means the field in the model register usuario
	#usuario_creador = fields.ForeignKey(UsuarioResource , 'usuario_creador')

	de = fields.ForeignKey(UsuarioResource,  full = True , attribute = 'de')
	para = fields.ForeignKey(UsuarioResource, 'para'    , full = True )
	status_id = fields.ForeignKey(StatusResource , 'status_id' ,  full =True )
	cc = fields.ToManyField('apps.api.resource.PreguntaCCResource',
		
		attribute = lambda bundle:	 PreguntaCc.objects.filter(pregunta_id=bundle.obj)
	,
	null=True,
	full = True,
)

	informacion = fields.CharField(attribute='informacion')
	class Meta:
		always_return_data = True		
		queryset = Pregunta.objects.all().order_by('-creacion')
		resource_name = 'pregunta'
		authorization= Authorization()

		filtering = {
			
			#'para' : ["exact"],
			'de' : ["exact"],
			'folio' : ["exact"],
			'pregunta' : ['contains'],
			'status_id' : ['exact'],
			'informacion' : ['contains'],

		}

 	def dehydrate(self , bundle ):
		if bundle.request.GET.get("para"):
			del bundle.data["para"]
			del bundle.data["cc"]
			del bundle.data["informacion"]

		bundle.data["status"] = bundle.data["status_id"].data["tipo"]
		del bundle.data["status_id"]
  		return bundle


	def obj_create(self , bundle , request = None , ):
#print bundle.obj.__dict__  bundle.obj directamente altera los nombres de las tablas de los modelos **les aumenta __id a los campos related
		bundle = self.full_hydrate(bundle)
		cc_ids = []
		cc = bundle.data['cc']

		for cc_id in cc:
			m = re.search('\/api\/v1\/usuario\/(\d+)\/', str(cc_id))			

			try:
				id = m.group(1)
				cc_ids.append(id)

			except AttributeError:
				pass

		user_cc = User.objects.filter(id__in = cc_ids )
		bundle.obj.save()

		#guardamos los CC en su prpia table
		for new_cc_users in user_cc:
			
			PreguntaCc.objects.create(   pregunta_id = bundle.obj, pregunta_cc = new_cc_users    )

		return bundle
	
	def get_object_list(self , request):

		try:

			user_id =  int(request.GET.get('for'))
			#se esta tratamdo de sacar tpdps las preguntas que se estan con copia
			print user_id

			questions_where_user_is_cc = PreguntaCc.objects.select_related().filter(pregunta_cc = user_id ).values("pregunta_id__id")
			print questions_where_user_is_cc
			#filtramos todas las preguntas para el usuario , "preguntas recibidas" incluyendo si se le ha enviado C.C
			questions_receive_= super(PreguntaResource, self).get_object_list(request).filter(Q( id__in  =  questions_where_user_is_cc) | Q( para = user_id ) )
			print questions_receive_
		 	return questions_receive_ 

		except:
			return super(PreguntaResource , self).get_object_list(request).all()


# C.C de la pregunta
class PreguntaCCResource(ModelResource):
	"""
    Return - Todos los usuarios que han sido asignados a alguna pregunta (C.C)
    pregunta_id -- ID de la pregunta 
    """
	pregunta_id = fields.ForeignKey(PreguntaResource , 'pregunta_id'  )
	pregunta_cc = fields.ForeignKey(UsuarioResource , 'pregunta_cc' , full= True)
 
	class Meta:
		#allowed_methods = ['get', 'delete', 'post', 'put']		
		queryset = PreguntaCc.objects.all()
		resource_name = 'cc'

		filtering = {
		#IMPORTANT pregunta_id is the table name no the object relation
			'pregunta_id' : ['exact'] , #access filter relation with settings
		 
			#'pub_date': ['exact', 'lt', 'lte', 'gte', 'gt'], #filters available
		
		}
		authorization= Authorization()

# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ******************************************* REPLY API **************************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************

class RespuestaCCResource(ModelResource):
	"""
    Return - Todas las respuestas de una pregunta  
    pregunta_id -- ID de la pregunta 
    """
 
	respuesta = fields.CharField(attribute='respuesta')
	creacion = fields.CharField(attribute='creacion')
	user = fields.ForeignKey(UsuarioResource,  full = True , attribute = 'user_creador')
	pregunta_id = fields.ForeignKey(PreguntaResource , 'pregunta_id'     )

	class Meta:
		#allowed_methods = ['get', 'delete', 'post', 'put']		
		queryset = Respuesta.objects.all().order_by("-creacion")
		resource_name = 'respuesta'
		always_return_data = True 
		filtering = {
			'pregunta_id' : ['exact'] , #access filter relation with settings
		}
		authorization= Authorization()

	def dehydrate(self , bundle ):
		
		bundle.data["first_name"] =  bundle.data["user"].data["first_name"]
		bundle.data["img"] =  bundle.data["user"].data["img"]
		bundle.data["last_name"] =  bundle.data["user"].data["last_name"]
		bundle.data["resource_uri"] =  bundle.data["user"].data["resource_uri"]
		bundle.data["username"] =  bundle.data["user"].data["username"]
		del bundle.data["user"]

		return bundle




# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ******************************************* TIMELINE API **************************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************
# http://127.0.0.1:8000/api/v1/timeline/?user_id=3&format=json      API CALL GET
# status_id=2 sera un parametro para filtrar las preguntas del timeline por status de la pregunta

class TimelineResource(ModelResource):
	
	  """
	  Return - Todos las preguntas realizadas por todos los usuarios a los que esta suscrito
	  el parametro @suscriptor
	  """		
          de = fields.ForeignKey(UsuarioResource, 'de' , full = True)
	  para = fields.ForeignKey(UsuarioResource, 'para' ,  full = True)
	  status_id = fields.ForeignKey(StatusResource , 'status_id' , full = True)
	  class Meta:
	 
	    queryset =  Pregunta.objects.all().order_by('-creacion')
	    resource_name = 'timeline'
	    filtering = {

		    "status_id"  : ["exact"] ,
		   }
          
	  def dehydrate(self , bundle):

		bundle.data["status"] = bundle.data["status_id"].data["tipo"]
		del bundle.data["status_id"]
		return bundle 
 
	  def get_object_list(self , request):

		try:

			user_id =  int(request.GET.get('user_id'))

			handler = Handlers()
			allow_users = handler.get_allow_contacts(user_id)
			allow_users_contacts = allow_users.values("id")
			
			user_timeline_followers_questions = super(TimelineResource, self).get_object_list(request).filter(de__in = allow_users_contacts )
		 	return user_timeline_followers_questions 

		except:
			return super(TimelineResource , self).get_object_list(request).all()

# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ******************************************* TIMELINE API **************************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************


class UserSignUpResource(ModelResource):

  class Meta:
    object_class = User
    queryset = User.objects.all()
    allowed_methods = ['post']
    include_resource_uri = False
    resource_name = 'newuser'
    excludes = ['is_active', 'is_staff', 'is_superuser','date_joined' ,'last_login']
    authorization= Authorization()
    always_return_data = True
    
  def dehydrate(self , bundle):
		
	setting = Settings.objects.filter(user_id = bundle.obj )
	bundle.data['img'] =	setting[0].img
	del bundle.data["password"]
#	bundle.data['rol'] = setting[0].rol
	return bundle

  def obj_create(self, bundle, request=None, **kwargs):

	error_message = ''

	try:
		bundle = self.full_hydrate(bundle)
		email = bundle.data.get('email')
		check_if_exist = User.objects.filter(email=email)

		if check_if_exist:
			raise BadRequest()
	except:
		
		error_message = 'El email ya existe'
	else:

		check_if_existe_username = User.objects.filter(username = bundle.data.get("username") )

		if check_if_existe_username:
			error_message = 'El usuario ya existe'
			raise BadRequest()
		
		rol_get = bundle.data.get("rol")
		bundle.obj.set_password(bundle.data.get('password'))
		user = bundle.obj.save()
		id_new_user =  bundle.obj.id
		rol = Rol.objects.filter(id = rol_get )[0]
		data= Settings.objects.create( user_id =  id_new_user , img = PATH_URL_IMG , rol = rol ) 

	finally:

		if error_message:
			raise BadRequest(error_message)
	return bundle

# ***********************************************************************************************************************
# ***********************************************************************************************************************
# ******************************************* ORGANIZACIONES API ********************************************************
# ***********************************************************************************************************************
# ***********************************************************************************************************************

#organizacion
class OrganizacionResource(ModelResource):
	
        creador = fields.ForeignKey(UsuarioResource, 'creador' , full = True)

	class Meta:
		always_return_data = True		
		queryset = Organizacion.objects.all().order_by("nombre")
		resource_name = 'organization'
		authorization= Authorization()

		limit = 0
		filtering = {
		

		}

	def dehydrate(self ,bundle):
		
		bundle.data["miembros"] = len( organizacion_usuario.objects.filter( organizacion = bundle.obj ) )
		return bundle


	def obj_create(self , bundle , request = None , ):
		print "backend"
		cc_ids = []
		cc = bundle.data['miembros']
		#nombre_organizacion = bundle.data['nombre']
		#new_user es una variable que si es declarada entonces quiere decir que se agregaran usaurios a una organizacion existente
		new_users = str(bundle.data['new_user'])

		print type(new_users)
		if new_users == 'true':
			#agregar usuarios a la organizacion
			print "nuevos usuarios"
			organization_id = bundle.data['organizacion_id']

			organizacion_ = Organizacion.objects.all().filter(id = organization_id )[0]
			print organizacion_ 
			bundle.obj = organizacion_
		else:
			#guardamos la nueva organizacion
			bundle = self.full_hydrate(bundle)
			bundle.obj.save()

		for cc_id in cc:
			m = re.search('\/api\/v1\/usuario\/(\d+)\/', str(cc_id))			

			try:
				id = m.group(1)
				cc_ids.append(id)

			except AttributeError:
				pass

		print cc_ids 

		user_cc = User.objects.filter(id__in = cc_ids )

		print user_cc
		#guardamos los nuevos miembros en la organizacion
		#guardamos solo los que no existen
		for new_cc_users in user_cc:
			
			organizacion_usuario.objects.get_or_create(   organizacion = bundle.obj, miembro = new_cc_users    )

		return bundle


#la organizacion tiene usuarios
class OrganizacionHasUserResource(ModelResource):

	organizacion = fields.ForeignKey(OrganizacionResource, 'organizacion'    , full = False)
        miembro = fields.ForeignKey(UsuarioResource, 'miembro' , full = True)

	def dehydrate(self , bundle):

		del bundle.data["miembro"].data["id"]
		bundle.data["first_name"] = bundle.data["miembro"].data["first_name"]
		bundle.data["img"] = bundle.data["miembro"].data["img"]
		bundle.data["last_name"] = bundle.data["miembro"].data["last_name"]
		bundle.data["username"] = bundle.data["miembro"].data["username"]
		del bundle.data["miembro"]

		return bundle
	class Meta:
		always_return_data = True		
		queryset = organizacion_usuario.objects.all()

		resource_name = 'organizacion_usuario'

		filtering = {
			
			"organizacion" : ["exact"],

		}
		authorization= Authorization()

		

